@@ -27,7 +27,8 @@ class BaseProtocol:
         sock = kwargs.get('sock')
         self.timeout = kwargs.get('timeout', 1)
         self.raise_timeout = kwargs.get('raise_timeout', False)
-        self.raise_incompleteread = kwargs.get("raise_incomplete_read",False)        
+        self.raise_incompleteread = kwargs.get("raise_incomplete_read",False)
+        self.max_try = kwargs.get("max_try",3600)        
         self.host = kwargs.get('host')
         self.port = kwargs.get('port')
         self.sock = self.create_socket(None)
@@ -79,6 +80,8 @@ class BaseProtocol:
         address = (host, port)
         counter = 0
         while not self.status:
+            if counter > self.max_try:
+                counter = 0
             try:
                 future_open_conn = asyncio.open_connection(
                     loop=loop,
@@ -86,12 +89,12 @@ class BaseProtocol:
                     port=port)  
                 (reader, writer) = await asyncio.wait_for(future_open_conn, timeout=self.timeout)
                 idc = await self.set_reader_writer(reader, writer)
-                if self.heart_beat(idc):
+                hb = await self.heart_beat(idc)
+                if hb:
                     self.status = True
                     self.logger.info("Conexion a %s realizada" % self.station)
                 else:
-                    heartbeat = self.heart_beat(idc)
-                    self.logger.error(heartbeat)
+                    self.logger.error(hb)
             except asyncio.TimeoutError as te:
                 self.logger.exception("Tiempo fuera en intento de conexión %s, iteracion %d" %(te, counter))
                 counter += 1
@@ -125,6 +128,13 @@ class BaseProtocol:
                     counter += 1                    
                     await asyncio.sleep(10+counter)
                 continue
+            except ConnectionResetError, ConnectionAbortedError as conn_error:
+                self.logger("Excepción por desconexión %s"%conn_error)
+                continue                
+            except Exception as e:
+                print("Excepción no considerada")
+                self.logger("Excepción no considerada %s"%e)
+                continue
             await asyncio.sleep(10+counter)
 
         return idc
@@ -132,7 +142,7 @@ class BaseProtocol:
     # callback for create server:
 
 
-    def heart_beat(self, idc):
+    async def heart_beat(self, idc):
         tnow = now()
         if idc in self.clients.keys():
             reader = self.clients[idc]['reader']
@@ -148,12 +158,12 @@ class BaseProtocol:
                 print(msg_error)
                 self.logger.error(msg_error)
                 self.logger.error("no heart_beat, at %s" %tnow)
-                self.close(idc)
+                await self.close(idc)
                 self.status = False
                 return False
         else:
             self.logger.error("no heart_beat, idc %s not client, code station %s" % (idc, self.station))
-            self.close(idc)
+            await self.close(idc)
             self.status = False
             return False
  
@@ -216,11 +226,19 @@ class BaseProtocol:
             self.logger.exception("Station %s, Tiempo fuera al leer en readbytes, %s"%(self.station,te))
             if self.raise_timeout:
                 raise te
+            else:
+                return None
         except asyncio.IncompleteReadError as ir:
             print("Incomplete read, station %s" %self.station, reader, n)
             self.logger.exception("Station %s, Tiempo fuera al no poder leer en readbytes %s bytes, %s"%(n, self.station,ir))
-            raise ir
-            
+            return None
+        except ConnectionResetError, ConnectionAbortedError as conn_error:
+            self.logger("Excepción por desconexión al intentar leer %s"%conn_error)
+            continue                
+        except Exception as e:
+            print("Excepción no considerada")
+            self.logger("Excepción no considerada al intentar leer%s"%e)
+            continue
 
     async def get_message_header(self, idc):
         self.idc = idc
